use crate::parser::template;
use crate::parser::Template;
use nom::IResult;
use std::collections::BTreeMap;

#[derive(Clone, Debug)]
pub struct CompiledTemplate<'a> {
    template: Template<'a>,
    name: String,
}

#[derive(Clone, Debug)]
pub struct DustContext<'a> {
    templates: BTreeMap<&'a String, &'a Template<'a>>,
}

pub fn compile_template<'a>(source: &'a str, name: String) -> CompiledTemplate<'a> {
    // TODO: Make this function return a result with a custom error type. Break the nom IResult chain
    // TODO: Make this all consuming
    let (_remaining, parsed_template) = template(source).expect("Failed to compile template");
    CompiledTemplate {
        template: parsed_template,
        name: name,
    }
}

impl<'a> DustContext<'a> {
    pub fn new() -> DustContext<'a> {
        DustContext {
            templates: BTreeMap::new(),
        }
    }

    pub fn load_source(&mut self, template: &'a CompiledTemplate) {
        self.templates.insert(&template.name, &template.template);
    }
}
